import os
from collections import defaultdict

from lab.reports import Table
from downward.reports import PlanningReport


class PlanComparer(PlanningReport):
    def __call__(self, eval_dir, outfile):
        self.eval_dir = os.path.abspath(eval_dir)
        self.outfile = os.path.abspath(outfile)

        self._load_data()
        self._apply_filter()
        self._scan_data()

        self.write()

    def get_markup(self):

        class Plan:
            def __init__(self, plan_string):
                self.plan = (plan_string.split("\n"))[:-2]

            def __str__(self):
                return "-".join(self.plan)


        # map plan strings to comparable plans
        new_props = {}
        for key, val in self.props.items():
            plan = val.get("plan")
            if plan is not None:
                val["plan"] = Plan(plan)
                new_props[key] = val

        # sort by problem
        runs_sorted = sorted(self.props.items(), key=lambda x: x[1].get("problem"), reverse=False)

        # group runs by problem
        grouped_problems = defaultdict(list)
        for key, val in runs_sorted:
            grouped_problems[val.get("problem")].append(val)

        # filter runs that didn't find a plan
        print()
        for problem, runs in grouped_problems.items():
            new_runs = list(filter(lambda x: x.get("plan") is not None, runs))
            grouped_problems[problem] = new_runs



        # compare plans of each problem
        # TODO: subdivide by domains not only algorithms

        tables = []

        for problem, runs in list(grouped_problems.items()):
            # dictionary containing mappings from plan to algorithm
            plan_dict = defaultdict(list)
            table = Table(title=problem)
            for run in runs:
                plan = run.get("plan")
                alg = run.get("algorithm")

                # convert plan to set of actions because order is irrelevant
                actions = set(plan.plan)

                plan_dict[str(actions)].append(alg)

            for (actions, algs) in plan_dict.items():
                row = {}

                row["actions"] = actions
                table.add_row(" | ".join(algs), row)

            tables.append(str(table))


        table_string = "\n\n".join(tables)

        return table_string